package accountCC

import (
	"fmt"
	"errors"
)

// AccountInfo account information with wallet id and account id
type TxnAccountInfo struct {
	WalletID  string `json:"wallet_id,omitempty"`
	AccountID string `json:"account_id,omitempty"`
}

// UserAccountInfo cpatures account info,
// including accountID, userID and walletID
type UserAccountInfo struct {
	UserID string `json:"user_id,omitempty"`
	// Embed account information into struct
	// with walletID and accountID
	*TxnAccountInfo
}

// String returns string representation of the account information
func (info UserAccountInfo) Validate() error {
	if len(info.UserID) == 0 {
		return errors.New("UserID can't be empty")
	}

	if info.TxnAccountInfo == nil {
		return errors.New("TxnAccountInfo can't be nil")
	}

	err := info.TxnAccountInfo.Validate()
	if err != nil {
		return err
	}

	return nil
}

// String returns string representation of the account information
func (info TxnAccountInfo) Validate() error {
	if len(info.WalletID) == 0 {
		return errors.New("WalletID can't be empty")
	}

	if len(info.AccountID) == 0 {
		return errors.New("AccountID can't be empty")
	}

	return nil
}

// String returns string representation of the account information
func (info UserAccountInfo) Get() *UserAccountInfo {
	return &UserAccountInfo{
		// Keep default values for time being, has to be replaced with
		// call to system chaincode to obtain system user and account information
		UserID: info.UserID,
		TxnAccountInfo: &TxnAccountInfo{
			WalletID:  info.WalletID,
			AccountID: info.AccountID,
		},
	}
}

// String returns string representation of the account information
func (info UserAccountInfo) String() string {
	if info.TxnAccountInfo == nil {
		return fmt.Sprintf("account information: user id <%s>", info.UserID)
	}

	return fmt.Sprintf("account information: user id <%s>, wallet id <%s>, account id <%s>",
		info.UserID, info.TxnAccountInfo.WalletID, info.TxnAccountInfo.AccountID)
}

// InKeyForCurrency generate key for "in" direction based on account information
// given currency id
func (info TxnAccountInfo) InKeyForCurrency(currencyType string) (string, error) {
	if len(info.WalletID) == 0 {
		msg := "can't create key 'InKeyForCurrency [WalletID: %s, AccountID: %s, currencyType: %s]' because WalletID can't be empty"
		errmsg := fmt.Sprintf(msg, info.WalletID, info.AccountID, currencyType)
		return "", errors.New(errmsg)
	}
	if len(info.AccountID) == 0 {
		msg := "can't create key 'InKeyForCurrency [WalletID: %s, AccountID: %s, currencyType: %s]' because AccountID can't be empty"
		errmsg := fmt.Sprintf(msg, info.WalletID, info.AccountID, currencyType)
		return "", errors.New(errmsg)
	}
	if len(currencyType) == 0 {
		msg := "can't create key 'InKeyForCurrency [WalletID: %s, AccountID: %s, currencyType: %s]' because currencyType can't be empty"
		errmsg := fmt.Sprintf(msg, info.WalletID, info.AccountID, currencyType)
		return "", errors.New(errmsg)
	}

	return fmt.Sprintf("in.%s.%s", currencyType, info.AccountID), nil
}

// OutKeyForCurrency generate key for "out" direction based on account information
// given currency id
func (info TxnAccountInfo) OutKeyForCurrency(currencyType string) (string, error) {
	if len(info.WalletID) == 0 {
		msg := "can't create key 'OutKeyForCurrency [WalletID: %s, AccountID: %s, currencyType: %s]' because WalletID can't be empty"
		errmsg := fmt.Sprintf(msg, info.WalletID, info.AccountID, currencyType)
		return "", errors.New(errmsg)
	}
	if len(info.AccountID) == 0 {
		msg := "can't create key 'OutKeyForCurrency [WalletID: %s, AccountID: %s, currencyType: %s]' because AccountID can't be empty"
		errmsg := fmt.Sprintf(msg, info.WalletID, info.AccountID, currencyType)
		return "", errors.New(errmsg)
	}
	if len(currencyType) == 0 {
		msg := "can't create key 'OutKeyForCurrency [WalletID: %s, AccountID: %s, currencyType: %s]' because currencyType can't be empty"
		errmsg := fmt.Sprintf(msg, info.WalletID, info.AccountID, currencyType)
		return "", errors.New(errmsg)
	}

	return fmt.Sprintf("out.%s.%s", currencyType, info.AccountID), nil
}
